@@ -16,10 +16,12 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "binary_buffer.h"
 #include "debug_info.h"
 #include "elf_file.h"
 #include "error.h"
 #include "linux_kernel.h"
+#include "platform.h"
 #include "program.h"
 #include "util.h"
 
@@ -364,7 +366,9 @@ drgn_debug_info_report_module(struct drgn_debug_info_load_state *load,
 		*new_ret = false;
 
 	struct hash_pair hp;
-	struct drgn_module_table_iterator it;
+	// Silence -Wmaybe-uninitialized false positive last seen with GCC 12 on
+	// i386 and Arm.
+	struct drgn_module_table_iterator it = {};
 	if (build_id_len) {
 		struct drgn_module_key key = {
 			.build_id = build_id,
@@ -417,6 +421,7 @@ drgn_debug_info_report_module(struct drgn_debug_info_load_state *load,
 		err = &drgn_enomem;
 		goto free;
 	}
+	module->prog = load->dbinfo->prog;
 	module->state = DRGN_DEBUG_INFO_MODULE_NEW;
 	module->build_id = build_id;
 	module->build_id_len = build_id_len;
