@@ -16,16 +16,16 @@
 #include <elfutils/version.h>
 #include <libelf.h>
 
-#include "binary_buffer.h"
 #include "cfi.h"
 #include "drgn.h"
 #include "dwarf_info.h"
 #include "hash_table.h"
 #include "orc_info.h"
-#include "platform.h"
 #include "string_builder.h"
 #include "vector.h"
 
+struct drgn_elf_file;
+
 /**
  * @ingroup Internals
  *
@@ -59,6 +59,8 @@ enum drgn_module_state {
  * address range) is uniquely represented by a @ref drgn_module.
  */
 struct drgn_module {
+	struct drgn_program *prog;
+
 	/** @c NULL if the module does not have a build ID. */
 	const void *build_id;
 	/** Zero if the module does not have a build ID. */
