@@ -28,6 +28,10 @@ struct drgn_error *linux_helper_read_vm(struct drgn_program *prog,
 					uint64_t pgtable, uint64_t virt_addr,
 					void *buf, size_t count);
 
+struct drgn_error *linux_helper_follow_phys(struct drgn_program *prog,
+					    uint64_t pgtable,
+					    uint64_t virt_addr, uint64_t *ret);
+
 struct drgn_error *linux_helper_per_cpu_ptr(struct drgn_object *res,
 					    const struct drgn_object *ptr,
 					    uint64_t cpu);
