@@ -25,6 +25,7 @@
 #include "helpers.h"
 #include "io.h"
 #include "linux_kernel.h"
+#include "platform.h"
 #include "program.h"
 #include "type.h"
 #include "util.h"
@@ -213,6 +214,43 @@ linux_kernel_get_uts_release(struct drgn_program *prog, struct drgn_object *ret)
 					   0, 0);
 }
 
+// jiffies is defined as an alias of jiffies_64 via the Linux kernel linker
+// script, so it is not included in debug info.
+static struct drgn_error *linux_kernel_get_jiffies(struct drgn_program *prog,
+						   struct drgn_object *ret)
+{
+	struct drgn_error *err;
+	struct drgn_object jiffies_64;
+	drgn_object_init(&jiffies_64, prog);
+	err = drgn_program_find_object(prog, "jiffies_64", NULL,
+				       DRGN_FIND_OBJECT_VARIABLE, &jiffies_64);
+	if (err) {
+		if (err->code == DRGN_ERROR_LOOKUP) {
+			drgn_error_destroy(err);
+			err = &drgn_not_found;
+		}
+		goto out;
+	}
+	if (jiffies_64.kind != DRGN_OBJECT_REFERENCE) {
+		err = &drgn_not_found;
+		goto out;
+	}
+	uint64_t address = jiffies_64.address;
+	struct drgn_qualified_type qualified_type;
+	err = drgn_program_find_primitive_type(prog, DRGN_C_TYPE_UNSIGNED_LONG,
+					       &qualified_type.type);
+	if (err)
+		return err;
+	qualified_type.qualifiers = DRGN_QUALIFIER_VOLATILE;
+	if (drgn_type_size(qualified_type.type) == 4 &&
+	    !drgn_type_little_endian(qualified_type.type))
+		address += 4;
+	err = drgn_object_set_reference(ret, qualified_type, address, 0, 0);
+out:
+	drgn_object_deinit(&jiffies_64);
+	return err;
+}
+
 // The vmemmap address can vary depending on architecture, kernel version,
 // configuration options, and KASLR. However, we can get it generically from the
 // section_mem_map of any valid mem_section.
@@ -320,7 +358,9 @@ static struct drgn_error *linux_kernel_get_vmemmap(struct drgn_program *prog,
 {
 	struct drgn_error *err;
 	if (prog->vmemmap.kind == DRGN_OBJECT_ABSENT) {
-		uint64_t address;
+		// Silence -Wmaybe-uninitialized false positive last seen with
+		// GCC 13 by initializing to zero.
+		uint64_t address = 0;
 		err = linux_kernel_get_vmemmap_address(prog, &address);
 		if (err)
 			return err;
@@ -458,12 +498,28 @@ kernel_module_iterator_next(struct kernel_module_iterator *it)
 		return err;
 
 	// Set tmp1 to the module base address and tmp2 to the size.
-	err = drgn_object_member(&it->tmp1, &it->mod, "core_layout");
+	err = drgn_object_member(&it->tmp1, &it->mod, "mem");
 	if (!err) {
-		// Since Linux kernel commit 7523e4dc5057 ("module: use a
-		// structure to encapsulate layout.") (in v4.5), the base and
-		// size are in the `struct module_layout core_layout` member of
-		// `struct module`.
+		union drgn_value mod_mem_type;
+
+		// Since Linux kernel commit ac3b43283923 ("module: replace
+		// module_layout with module_memory") (in v6.4), the base and
+		// size are in the `struct module_memory mem[MOD_TEXT]` member
+		// of `struct module`.
+		err = drgn_program_find_object(drgn_object_program(&it->mod),
+					       "MOD_TEXT", NULL,
+					       DRGN_FIND_OBJECT_CONSTANT,
+					       &it->tmp2);
+		if (err)
+			return err;
+		err = drgn_object_read_integer(&it->tmp2, &mod_mem_type);
+		if (err)
+			return err;
+
+		err = drgn_object_subscript(&it->tmp1, &it->tmp1,
+					    mod_mem_type.uvalue);
+		if (err)
+			return err;
 		err = drgn_object_member(&it->tmp2, &it->tmp1, "size");
 		if (err)
 			return err;
@@ -471,15 +527,36 @@ kernel_module_iterator_next(struct kernel_module_iterator *it)
 		if (err)
 			return err;
 	} else if (err->code == DRGN_ERROR_LOOKUP) {
-		// Before that, they are directly in the `struct module`.
+		// Since Linux kernel commit 7523e4dc5057 ("module: use a
+		// structure to encapsulate layout.") (in v4.5), the base and
+		// size are in the `struct module_layout core_layout` member of
+		// `struct module`.
 		drgn_error_destroy(err);
 
-		err = drgn_object_member(&it->tmp2, &it->mod, "core_size");
-		if (err)
-			return err;
-		err = drgn_object_member(&it->tmp1, &it->mod, "module_core");
-		if (err)
+		err = drgn_object_member(&it->tmp1, &it->mod, "core_layout");
+		if (!err) {
+			err = drgn_object_member(&it->tmp2, &it->tmp1, "size");
+			if (err)
+				return err;
+			err = drgn_object_member(&it->tmp1, &it->tmp1, "base");
+			if (err)
+				return err;
+		} else if (err->code == DRGN_ERROR_LOOKUP) {
+			// Before that, they are directly in the `struct
+			// module`.
+			drgn_error_destroy(err);
+
+			err = drgn_object_member(&it->tmp2, &it->mod,
+						 "core_size");
+			if (err)
+				return err;
+			err = drgn_object_member(&it->tmp1, &it->mod,
+						 "module_core");
+			if (err)
+				return err;
+		} else {
 			return err;
+		}
 	} else {
 		return err;
 	}
