@@ -6,6 +6,7 @@ import ctypes
 import mmap
 import os
 import struct
+import sys
 import tempfile
 import unittest
 
@@ -26,6 +27,9 @@ from drgn.helpers.linux.mm import (
     compound_order,
     decode_page_flags,
     environ,
+    follow_page,
+    follow_pfn,
+    follow_phys,
     page_size,
     page_to_pfn,
     page_to_phys,
@@ -34,14 +38,18 @@ from drgn.helpers.linux.mm import (
     pfn_to_virt,
     phys_to_page,
     phys_to_virt,
+    totalram_pages,
     virt_to_page,
     virt_to_pfn,
     virt_to_phys,
+    vmalloc_to_page,
+    vmalloc_to_pfn,
 )
 from drgn.helpers.linux.pid import find_task
 from tests.linux_kernel import (
     LinuxKernelTestCase,
     mlock,
+    prng32,
     skip_unless_have_full_mm_support,
     skip_unless_have_test_kmod,
 )
@@ -142,28 +150,24 @@ class TestMm(LinuxKernelTestCase):
             page = pfn_to_page(self.prog, pfns[0])
             self.assertIn("PG_swapbacked", decode_page_flags(page))
 
-    @skip_unless_have_full_mm_support
     @skip_unless_have_test_kmod
     def test_PFN_PHYS(self):
         self.assertEqual(
             PFN_PHYS(self.prog["drgn_test_pfn"]), self.prog["drgn_test_pa"]
         )
 
-    @skip_unless_have_full_mm_support
     @skip_unless_have_test_kmod
     def test_PHYS_PFN(self):
         self.assertEqual(
             PHYS_PFN(self.prog["drgn_test_pa"]), self.prog["drgn_test_pfn"]
         )
 
-    @skip_unless_have_full_mm_support
     @skip_unless_have_test_kmod
     def test_page_to_pfn(self):
         self.assertEqual(
             page_to_pfn(self.prog["drgn_test_page"]), self.prog["drgn_test_pfn"]
         )
 
-    @skip_unless_have_full_mm_support
     @skip_unless_have_test_kmod
     def test_page_to_phys(self):
         self.assertEqual(
@@ -177,7 +181,6 @@ class TestMm(LinuxKernelTestCase):
             page_to_virt(self.prog["drgn_test_page"]), self.prog["drgn_test_va"]
         )
 
-    @skip_unless_have_full_mm_support
     @skip_unless_have_test_kmod
     def test_pfn_to_page(self):
         self.assertEqual(
@@ -191,7 +194,6 @@ class TestMm(LinuxKernelTestCase):
             pfn_to_virt(self.prog["drgn_test_pfn"]), self.prog["drgn_test_va"]
         )
 
-    @skip_unless_have_full_mm_support
     @skip_unless_have_test_kmod
     def test_phys_to_page(self):
         self.assertEqual(
@@ -226,13 +228,59 @@ class TestMm(LinuxKernelTestCase):
             virt_to_phys(self.prog["drgn_test_va"]), self.prog["drgn_test_pa"]
         )
 
+    @skip_unless_have_test_kmod
     def test_read_physical(self):
-        with self._pages() as (map, _, pfns):
-            for i, pfn in enumerate(pfns):
-                self.assertEqual(
-                    self.prog.read(pfn * mmap.PAGESIZE, mmap.PAGESIZE, True),
-                    map[i * mmap.PAGESIZE : (i + 1) * mmap.PAGESIZE],
-                )
+        expected = bytearray()
+        for x in prng32("PAGE"):
+            expected.extend(x.to_bytes(4, sys.byteorder))
+            if len(expected) >= mmap.PAGESIZE:
+                break
+        self.assertEqual(
+            self.prog.read(self.prog["drgn_test_pa"], mmap.PAGESIZE, True), expected
+        )
+
+    @skip_unless_have_full_mm_support
+    @skip_unless_have_test_kmod
+    def test_follow_phys(self):
+        self.assertEqual(
+            follow_phys(self.prog["init_mm"].address_of_(), self.prog["drgn_test_va"]),
+            self.prog["drgn_test_pa"],
+        )
+
+    @skip_unless_have_full_mm_support
+    @skip_unless_have_test_kmod
+    def test_follow_page(self):
+        self.assertEqual(
+            follow_page(self.prog["init_mm"].address_of_(), self.prog["drgn_test_va"]),
+            self.prog["drgn_test_page"],
+        )
+
+    @skip_unless_have_full_mm_support
+    @skip_unless_have_test_kmod
+    def test_follow_pfn(self):
+        self.assertEqual(
+            follow_pfn(self.prog["init_mm"].address_of_(), self.prog["drgn_test_va"]),
+            self.prog["drgn_test_pfn"],
+        )
+        task = find_task(self.prog, os.getpid())
+        with self._pages() as (map, address, pfns):
+            self.assertEqual(follow_pfn(task.mm, address), pfns[0])
+
+    @skip_unless_have_full_mm_support
+    @skip_unless_have_test_kmod
+    def test_vmalloc_to_page(self):
+        self.assertEqual(
+            vmalloc_to_page(self.prog["drgn_test_vmalloc_va"]),
+            self.prog["drgn_test_vmalloc_page"],
+        )
+
+    @skip_unless_have_full_mm_support
+    @skip_unless_have_test_kmod
+    def test_vmalloc_to_pfn(self):
+        self.assertEqual(
+            vmalloc_to_pfn(self.prog["drgn_test_vmalloc_va"]),
+            self.prog["drgn_test_vmalloc_pfn"],
+        )
 
     @skip_unless_have_full_mm_support
     def test_access_process_vm(self):
@@ -291,3 +339,14 @@ class TestMm(LinuxKernelTestCase):
             proc_environ = f.read().split(b"\0")[:-1]
         task = find_task(self.prog, os.getpid())
         self.assertEqual(environ(task), proc_environ)
+
+    def test_totalram_pages(self):
+        with open("/proc/meminfo") as f:
+            lines = f.read().splitlines()
+            line = [line for line in lines if line.startswith("MemTotal:")][0]
+            parts = line.split()
+            self.assertEqual(parts[2], "kB")
+
+            proc_totalram = 1024 * int(parts[1])
+            page_size = self.prog["PAGE_SIZE"].value_()
+            self.assertEqual(totalram_pages(self.prog) * page_size, proc_totalram)
