@@ -22,6 +22,7 @@
 #include <linux/rbtree_augmented.h>
 #include <linux/slab.h>
 #include <linux/version.h>
+#include <linux/vmalloc.h>
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0)
 #define HAVE_XARRAY 1
 #include <linux/xarray.h>
@@ -29,6 +30,23 @@
 #define HAVE_XARRAY 0
 #endif
 
+// Convert a 4-character string to a seed for drgn_test_prng32().
+static inline u32 drgn_test_prng32_seed(const char *s)
+{
+	BUG_ON(strlen(s) != 4);
+	return ((u32)s[0] << 24) | ((u32)s[1] << 16) | ((u32)s[2] << 8) | (u32)s[3];
+}
+
+// x must not be 0; the return value is never 0.
+static u32 drgn_test_prng32(u32 x)
+{
+	// Xorshift RNG with a period of 2^32 - 1.
+	x ^= x << 13;
+	x ^= x >> 17;
+	x ^= x << 5;
+	return x;
+}
+
 // list
 
 LIST_HEAD(drgn_test_empty_list);
@@ -130,9 +148,15 @@ phys_addr_t drgn_test_pa;
 unsigned long drgn_test_pfn;
 struct page *drgn_test_page;
 struct page *drgn_test_compound_page;
+void *drgn_test_vmalloc_va;
+unsigned long drgn_test_vmalloc_pfn;
+struct page *drgn_test_vmalloc_page;
 
 static int drgn_test_mm_init(void)
 {
+	u32 fill;
+	size_t i;
+
 	drgn_test_page = alloc_page(GFP_KERNEL);
 	if (!drgn_test_page)
 		return -ENOMEM;
@@ -140,13 +164,25 @@ static int drgn_test_mm_init(void)
 	if (!drgn_test_compound_page)
 		return -ENOMEM;
 	drgn_test_va = page_address(drgn_test_page);
+	// Fill the page with a PRNG sequence.
+	fill = drgn_test_prng32_seed("PAGE");
+	for (i = 0; i < PAGE_SIZE / sizeof(fill); i++) {
+		fill = drgn_test_prng32(fill);
+		((u32 *)drgn_test_va)[i] = fill;
+	}
 	drgn_test_pa = virt_to_phys(drgn_test_va);
 	drgn_test_pfn = PHYS_PFN(drgn_test_pa);
+	drgn_test_vmalloc_va = vmalloc(PAGE_SIZE);
+	if (!drgn_test_vmalloc_va)
+		return -ENOMEM;
+	drgn_test_vmalloc_pfn = vmalloc_to_pfn(drgn_test_vmalloc_va);
+	drgn_test_vmalloc_page = vmalloc_to_page(drgn_test_vmalloc_va);
 	return 0;
 }
 
 static void drgn_test_mm_exit(void)
 {
+	vfree(drgn_test_vmalloc_va);
 	if (drgn_test_compound_page)
 		__free_pages(drgn_test_compound_page, 1);
 	if (drgn_test_page)
@@ -156,27 +192,24 @@ static void drgn_test_mm_exit(void)
 
 // percpu
 
-DEFINE_PER_CPU(unsigned int, drgn_test_percpu_static);
-const unsigned int drgn_test_percpu_static_prime = 0xa45dcfc3U;
-unsigned int __percpu *drgn_test_percpu_dynamic;
-const unsigned int drgn_test_percpu_dynamic_prime = 0x6d80a613U;
+DEFINE_PER_CPU(u32, drgn_test_percpu_static);
+u32 __percpu *drgn_test_percpu_dynamic;
 
 static int drgn_test_percpu_init(void)
 {
 	int cpu;
-	unsigned int static_prime = drgn_test_percpu_static_prime;
-	unsigned int dynamic_prime = drgn_test_percpu_dynamic_prime;
+	u32 static_seed = drgn_test_prng32_seed("PCPU");
+	u32 dynamic_seed = drgn_test_prng32_seed("pcpu");
 
-	drgn_test_percpu_dynamic = alloc_percpu(unsigned int);
+	drgn_test_percpu_dynamic = alloc_percpu(u32);
 	if (!drgn_test_percpu_dynamic)
 		return -ENOMEM;
-	// Initialize the per-cpu variables to powers of a random prime number
-	// which are extremely unlikely to appear anywhere else.
+	// Initialize the per-cpu variables with a PRNG sequence.
 	for_each_possible_cpu(cpu) {
-		static_prime *= drgn_test_percpu_static_prime;
-		per_cpu(drgn_test_percpu_static, cpu) = static_prime;
-		dynamic_prime *= drgn_test_percpu_dynamic_prime;
-		*per_cpu_ptr(drgn_test_percpu_dynamic, cpu) = dynamic_prime;
+		static_seed = drgn_test_prng32(static_seed);
+		per_cpu(drgn_test_percpu_static, cpu) = static_seed;
+		dynamic_seed = drgn_test_prng32(dynamic_seed);
+		*per_cpu_ptr(drgn_test_percpu_dynamic, cpu) = dynamic_seed;
 	}
 	return 0;
 }
@@ -739,6 +772,52 @@ static void drgn_test_xarray_exit(void)
 #endif
 }
 
+// idr
+
+DEFINE_IDR(drgn_test_idr_empty);
+DEFINE_IDR(drgn_test_idr_one);
+DEFINE_IDR(drgn_test_idr_one_at_zero);
+DEFINE_IDR(drgn_test_idr_sparse);
+
+static int drgn_test_idr_init(void)
+{
+	int ret;
+
+	ret = idr_alloc(&drgn_test_idr_one, (void *)0xdeadb00, 66, 67,
+			GFP_KERNEL);
+	if (ret < 0)
+		return ret;
+
+	ret = idr_alloc(&drgn_test_idr_one_at_zero, (void *)0x1234, 0, 1,
+			GFP_KERNEL);
+	if (ret < 0)
+		return ret;
+
+	ret = idr_alloc(&drgn_test_idr_sparse, (void *)0x1234, 1, 2,
+			GFP_KERNEL);
+	if (ret < 0)
+		return ret;
+
+	ret = idr_alloc(&drgn_test_idr_sparse, (void *)0x5678, 0x80, 0x81,
+			GFP_KERNEL);
+	if (ret < 0)
+		return ret;
+
+	ret = idr_alloc(&drgn_test_idr_sparse, (void *)0x9abc, 0xee, 0xef,
+			GFP_KERNEL);
+	if (ret < 0)
+		return ret;
+
+	return 0;
+}
+
+static void drgn_test_idr_exit(void)
+{
+	idr_destroy(&drgn_test_idr_one);
+	idr_destroy(&drgn_test_idr_one_at_zero);
+	idr_destroy(&drgn_test_idr_sparse);
+}
+
 // Dummy function symbol.
 int drgn_test_function(int x)
 {
@@ -753,6 +832,7 @@ static void drgn_test_exit(void)
 	drgn_test_stack_trace_exit();
 	drgn_test_radix_tree_exit();
 	drgn_test_xarray_exit();
+	drgn_test_idr_exit();
 }
 
 static int __init drgn_test_init(void)
@@ -778,6 +858,9 @@ static int __init drgn_test_init(void)
 	if (ret)
 		goto out;
 	ret = drgn_test_xarray_init();
+	if (ret)
+		goto out;
+	ret = drgn_test_idr_init();
 out:
 	if (ret)
 		drgn_test_exit();
