@@ -5,14 +5,15 @@ import argparse
 import asyncio
 import filecmp
 import logging
+import os
 from pathlib import Path
 import shlex
 import shutil
 import sys
 import tempfile
-from typing import IO, Any, NamedTuple, Optional, Tuple, Union
+from typing import IO, Any, Mapping, NamedTuple, Optional, Sequence, Tuple, Union
 
-from util import nproc
+from util import KernelVersion, nproc
 from vmtest.asynciosubprocess import (
     CalledProcessError,
     check_call,
@@ -20,169 +21,135 @@ from vmtest.asynciosubprocess import (
     check_output_shell,
     pipe_context,
 )
+from vmtest.config import (
+    ARCHITECTURES,
+    HOST_ARCHITECTURE,
+    KERNEL_FLAVORS,
+    Architecture,
+    Compiler,
+    KernelFlavor,
+    kconfig,
+    kconfig_localversion,
+)
+from vmtest.download import COMPILER_URL, DownloadCompiler, download
 
 logger = logging.getLogger(__name__)
 
 
-class KernelFlavor(NamedTuple):
-    name: str
-    description: str
-    config: str
-
-    def localversion(self) -> str:
-        localversion = "-vmtest16"
-        # The default flavor should be the "latest" version.
-        localversion += ".1" if self.name == "default" else ".0"
-        localversion += self.name
-        return localversion
-
-
-KERNEL_FLAVORS = [
-    KernelFlavor(
-        name="default",
-        description="Default configuration",
-        config="""
-CONFIG_SMP=y
-CONFIG_SLUB=y
-# For slab tests.
-CONFIG_SLUB_DEBUG=y
-""",
-    ),
-    KernelFlavor(
-        name="alternative",
-        description="SLAB allocator",
-        config="""
-CONFIG_SMP=y
-CONFIG_SLAB=y
-""",
-    ),
-    KernelFlavor(
-        name="tiny",
-        description="!SMP, !PREEMPT, and SLOB allocator",
-        config="""
-CONFIG_SMP=n
-CONFIG_SLOB=y
-# Linux kernel commit 149b6fa228ed ("mm, slob: rename CONFIG_SLOB to
-# CONFIG_SLOB_DEPRECATED") (in v6.2) renamed to option for SLOB.
-CONFIG_SLOB_DEPRECATED=y
-# CONFIG_PREEMPT_DYNAMIC is not set
-CONFIG_PREEMPT_NONE=y
-# !PREEMPTION && !SMP will also select TINY_RCU.
-""",
-    ),
-]
-
-
 _PACKAGE_FORMATS = ("tar.zst", "directory")
 
 
-def kconfig(flavor: KernelFlavor) -> str:
-    return rf"""# Minimal Linux kernel configuration for booting into vmtest and running drgn
-# tests ({flavor.name} flavor).
+class _Patch(NamedTuple):
+    name: str
+    # [inclusive, exclusive) ranges of kernel versions to apply this patch to.
+    # None means any version.
+    versions: Sequence[Tuple[Optional[KernelVersion], Optional[KernelVersion]]]
 
-CONFIG_LOCALVERSION="{flavor.localversion()}"
-CONFIG_EXPERT=y
-{flavor.config}
-CONFIG_MODULES=y
-CONFIG_MODULE_UNLOAD=y
-CONFIG_CC_OPTIMIZE_FOR_SIZE=y
 
-# We run the tests in KVM.
-CONFIG_HYPERVISOR_GUEST=y
-CONFIG_KVM_GUEST=y
-CONFIG_PARAVIRT=y
-CONFIG_PARAVIRT_SPINLOCKS=y
+_PATCHES = (
+    _Patch(
+        name="proc-kcore-allow-enabling-CONFIG_PROC_KCORE-on-ARM.patch",
+        versions=((None, None),),
+    ),
+    _Patch(
+        name="5.15-kbuild-Unify-options-for-BTF-generation-for-vmlinux.patch",
+        versions=((KernelVersion("5.13"), KernelVersion("5.15.66")),),
+    ),
+    _Patch(
+        name="5.12-kbuild-Quote-OBJCOPY-var-to-avoid-a-pahole-call-brea.patch",
+        versions=((KernelVersion("5.11"), KernelVersion("5.12.10")),),
+    ),
+    _Patch(
+        name="5.11-bpf-Generate-BTF_KIND_FLOAT-when-linking-vmlinux.patch",
+        versions=((KernelVersion("5.11"), KernelVersion("5.13")),),
+    ),
+    _Patch(
+        name="5.10-kbuild-skip-per-CPU-BTF-generation-for-pahole-v1.18-.patch",
+        versions=((KernelVersion("5.11"), KernelVersion("5.13")),),
+    ),
+    _Patch(
+        name="5.11-kbuild-Unify-options-for-BTF-generation-for-vmlinux.patch",
+        versions=((KernelVersion("5.11"), KernelVersion("5.13")),),
+    ),
+    _Patch(
+        name="kbuild-Add-skip_encoding_btf_enum64-option-to-pahole.patch",
+        versions=((KernelVersion("5.18"), KernelVersion("5.19.17")),),
+    ),
+    _Patch(
+        name="5.15-kbuild-Add-skip_encoding_btf_enum64-option-to-pahole.patch",
+        versions=(
+            (KernelVersion("5.16"), KernelVersion("5.18")),
+            (KernelVersion("5.13"), KernelVersion("5.15.66")),
+        ),
+    ),
+    _Patch(
+        name="5.10-kbuild-Add-skip_encoding_btf_enum64-option-to-pahole.patch",
+        versions=((KernelVersion("5.11"), KernelVersion("5.13")),),
+    ),
+    _Patch(
+        name="s390-mm-make-memory_block_size_bytes-available-for-M.patch",
+        versions=((KernelVersion("4.3"), KernelVersion("4.11")),),
+    ),
+    _Patch(
+        name="libsubcmd-Fix-use-after-free-for-realloc-.-0.patch",
+        versions=(
+            (KernelVersion("5.16"), KernelVersion("5.16.11")),
+            (KernelVersion("5.11"), KernelVersion("5.15.25")),
+            (KernelVersion("5.5"), KernelVersion("5.10.102")),
+            (KernelVersion("4.20"), KernelVersion("5.4.181")),
+            (KernelVersion("4.15"), KernelVersion("4.19.231")),
+            (KernelVersion("4.10"), KernelVersion("4.14.268")),
+            (KernelVersion("4.5"), KernelVersion("4.9.303")),
+        ),
+    ),
+)
 
-# Minimum requirements for vmtest.
-CONFIG_9P_FS=y
-CONFIG_DEVTMPFS=y
-CONFIG_INET=y
-CONFIG_NET=y
-CONFIG_NETWORK_FILESYSTEMS=y
-CONFIG_NET_9P=y
-CONFIG_NET_9P_VIRTIO=y
-CONFIG_OVERLAY_FS=y
-CONFIG_PCI=y
-CONFIG_PROC_FS=y
-CONFIG_SERIAL_8250=y
-CONFIG_SERIAL_8250_CONSOLE=y
-CONFIG_SYSFS=y
-CONFIG_TMPFS=y
-CONFIG_TMPFS_XATTR=y
-CONFIG_VIRTIO_CONSOLE=y
-CONFIG_VIRTIO_PCI=y
-CONFIG_HW_RANDOM=m
-CONFIG_HW_RANDOM_VIRTIO=m
 
-# drgn needs debug info.
-CONFIG_DEBUG_KERNEL=y
-CONFIG_DEBUG_INFO=y
-CONFIG_DEBUG_INFO_DWARF4=y
-
-# For testing live kernel debugging with /proc/kcore.
-CONFIG_PROC_KCORE=y
-# drgn needs /proc/kallsyms in some cases. Some test cases also need it.
-CONFIG_KALLSYMS=y
-CONFIG_KALLSYMS_ALL=y
-
-# For testing kernel core dumps with /proc/vmcore.
-CONFIG_CRASH_DUMP=y
-CONFIG_PROC_VMCORE=y
-CONFIG_KEXEC=y
-CONFIG_KEXEC_FILE=y
-# Needed for CONFIG_KEXEC_FILE.
-CONFIG_CRYPTO=y
-CONFIG_CRYPTO_SHA256=y
-
-# So that we can trigger a crash with /proc/sysrq-trigger.
-CONFIG_MAGIC_SYSRQ=y
-
-# For block tests.
-CONFIG_BLK_DEV_LOOP=m
-
-# For BPF tests.
-CONFIG_BPF_SYSCALL=y
-CONFIG_BPF_JIT=y
-CONFIG_BPF_JIT_ALWAYS_ON=y
-CONFIG_CGROUP_BPF=y
-CONFIG_DEBUG_INFO_BTF=y
-CONFIG_DEBUG_INFO_BTF_MODULES=y
-
-# For cgroup tests.
-CONFIG_CGROUPS=y
-
-# For kconfig tests.
-CONFIG_IKCONFIG=m
-CONFIG_IKCONFIG_PROC=y
-
-# For filesystem tests.
-CONFIG_BTRFS_FS=m
-CONFIG_EXT4_FS=m
-CONFIG_XFS_FS=m
-
-# For net tests.
-CONFIG_NAMESPACES=y
-
-# For nodemask tests.
-CONFIG_NUMA=y
-
-# For slab allocator tests.
-CONFIG_SLAB_FREELIST_HARDENED=y
-
-# For Traffic Control tests.
-CONFIG_NET_SCHED=y
-CONFIG_NET_SCH_PRIO=m
-CONFIG_NET_SCH_SFQ=m
-CONFIG_NET_SCH_TBF=m
-CONFIG_NET_SCH_INGRESS=m
-CONFIG_NET_CLS_ACT=y
-CONFIG_NETDEVICES=y
-CONFIG_DUMMY=m
-
-# To enable CONFIG_XARRAY_MULTI for xarray tests.
-CONFIG_TRANSPARENT_HUGEPAGE=y
-CONFIG_READ_ONLY_THP_FOR_FS=y
-"""
+async def apply_patches(kernel_dir: Path) -> None:
+    patch_dir = Path(__file__).parent / "patches"
+    version = KernelVersion(
+        (await check_output("make", "-s", "kernelversion", cwd=kernel_dir))
+        .decode()
+        .strip()
+    )
+    logging.info("applying patches for kernel version %s", version)
+    any_applied = False
+    for patch in _PATCHES:
+        for min_version, max_version in patch.versions:
+            if (min_version is None or min_version <= version) and (
+                max_version is None or version < max_version
+            ):
+                break
+        else:
+            continue
+        logging.info("applying %s", patch.name)
+        any_applied = True
+        proc = await asyncio.create_subprocess_exec(
+            "git",
+            "apply",
+            str(patch_dir / patch.name),
+            cwd=kernel_dir,
+            stderr=asyncio.subprocess.PIPE,
+        )
+        stderr = await proc.stderr.read()
+        if await proc.wait() != 0:
+            try:
+                await check_call(
+                    "git",
+                    "apply",
+                    "--reverse",
+                    "--check",
+                    str(patch_dir / patch.name),
+                    cwd=kernel_dir,
+                    stderr=asyncio.subprocess.DEVNULL,
+                )
+            except CalledProcessError:
+                sys.stderr.buffer.write(stderr)
+                sys.stderr.buffer.flush()
+                raise
+            logging.info("already applied")
+    if not any_applied:
+        logging.info("no patches")
 
 
 class KBuild:
@@ -190,15 +157,16 @@ class KBuild:
         self,
         kernel_dir: Path,
         build_dir: Path,
+        arch: Architecture,
         flavor: KernelFlavor,
-        arch: str,
+        env: Optional[Mapping[str, str]] = None,
         build_log_file: Union[int, IO[Any], None] = None,
     ) -> None:
         self._build_dir = build_dir
         self._kernel_dir = kernel_dir
         self._flavor = flavor
         self._arch = arch
-        self._srcarch = {"x86_64": "x86"}.get(arch, arch)
+        self._env = env
         self._build_stdout = build_log_file
         self._build_stderr = (
             None if build_log_file is None else asyncio.subprocess.STDOUT
@@ -242,7 +210,8 @@ class KBuild:
             self._cached_make_args = (
                 "-C",
                 str(self._kernel_dir),
-                "ARCH=" + str(self._arch),
+                "ARCH=" + str(self._arch.kernel_arch),
+                "LOCALVERSION=" + kconfig_localversion(self._flavor),
                 "O=" + str(build_dir_real),
                 "KBUILD_ABS_SRCTREE=1",
                 "KBUILD_BUILD_USER=drgn",
@@ -262,7 +231,11 @@ class KBuild:
             self._cached_kernel_release = (
                 (
                     await check_output(
-                        "make", *self._cached_make_args, "-s", "kernelrelease"
+                        "make",
+                        *self._cached_make_args,
+                        "-s",
+                        "kernelrelease",
+                        env=self._env,
                     )
                 )
                 .decode()
@@ -271,7 +244,12 @@ class KBuild:
         return self._cached_kernel_release
 
     async def build(self) -> None:
-        logger.info("building %s kernel in %s", self._flavor.name, self._build_dir)
+        logger.info(
+            "building %s %s kernel in %s",
+            self._arch.name,
+            self._flavor.name,
+            self._build_dir,
+        )
         build_log_file_name = getattr(self._build_stdout, "name", None)
         if build_log_file_name is not None:
             logger.info("build logs in %s", build_log_file_name)
@@ -281,7 +259,7 @@ class KBuild:
         config = self._build_dir / ".config"
         tmp_config = self._build_dir / ".config.vmtest.tmp"
 
-        tmp_config.write_text(kconfig(self._flavor))
+        tmp_config.write_text(kconfig(self._arch, self._flavor))
         await check_call(
             "make",
             *make_args,
@@ -289,6 +267,7 @@ class KBuild:
             "olddefconfig",
             stdout=self._build_stdout,
             stderr=self._build_stderr,
+            env=self._env,
         )
         try:
             equal = filecmp.cmp(config, tmp_config)
@@ -311,34 +290,32 @@ class KBuild:
             "all",
             stdout=self._build_stdout,
             stderr=self._build_stderr,
+            env=self._env,
+        )
+        logger.info(
+            "built kernel %s for %s in %s",
+            kernel_release,
+            self._arch.name,
+            self._build_dir,
         )
-        logger.info("built kernel %s in %s", kernel_release, self._build_dir)
 
-    def _copy_module_build(self, modules_dir: Path) -> None:
+    def _copy_module_build(self, modules_build_dir: Path) -> None:
         logger.info("copying module build files")
 
-        # `make modules_install` creates these as symlinks to the absolute path
-        # of the source directory. Delete them, populate build, and make source
-        # a symlink to build.
-        modules_build_dir = modules_dir / "build"
-        modules_source_dir = modules_dir / "source"
-        modules_build_dir.unlink()
-        modules_build_dir.mkdir()
-        modules_source_dir.unlink()
-        modules_source_dir.symlink_to("build")
-
         # Files and directories (as glob patterns) required for external module
         # builds. This list was determined through trial and error.
-        files = (
+        files = [
             ".config",
             "Module.symvers",
-            f"arch/{self._srcarch}/Makefile",
+            f"arch/{self._arch.kernel_srcarch}/Makefile",
+            f"arch/{self._arch.kernel_srcarch}/kernel/module.lds",
             "scripts/Kbuild.include",
             "scripts/Makefile*",
             "scripts/basic/fixdep",
             "scripts/check-local-export",
             "scripts/gcc-goto.sh",
             "scripts/gcc-version.sh",
+            "scripts/ld-version.sh",
             "scripts/mod/modpost",
             "scripts/module-common.lds",
             "scripts/module.lds",
@@ -346,10 +323,21 @@ class KBuild:
             "scripts/pahole-flags.sh",
             "scripts/pahole-version.sh",
             "scripts/subarch.include",
+            "tools/bpf/resolve_btfids/resolve_btfids",
             "tools/objtool/objtool",
-        )
+        ]
+        # Before Linux kernel commit bca8f17f57bd ("arm64: Get rid of
+        # asm/opcodes.h") (in v4.10), AArch64 includes this file from 32-bit
+        # Arm.
+        if self._arch.name == "aarch64":
+            files.append("arch/arm/include/asm/opcodes.h")
+        # Before Linux kernel commit efe0160cfd40 ("powerpc/64: Linker
+        # on-demand sfpr functions for modules") (in v4.13), this must be
+        # available to link into modules.
+        if self._arch.name == "ppc64":
+            files.append("arch/powerpc/lib/crtsavres.o")
         directories = (
-            f"arch/{self._srcarch}/include",
+            f"arch/{self._arch.kernel_srcarch}/include",
             "include",
         )
 
@@ -383,7 +371,7 @@ class KBuild:
                         dirs_exist_ok=True,
                     )
 
-    async def _test_external_module_build(self, modules_dir: Path) -> None:
+    async def _test_external_module_build(self, modules_build_dir: Path) -> None:
         logger.info("testing external module build")
 
         with tempfile.TemporaryDirectory(
@@ -411,12 +399,16 @@ MODULE_LICENSE("GPL");
             # missing some files.
             cmd = (
                 "make",
+                "ARCH=" + str(self._arch.kernel_arch),
                 "-C",
-                modules_dir / "build",
+                modules_build_dir,
                 f"M={test_module_dir.resolve()}",
             )
             proc = await asyncio.create_subprocess_exec(
-                *cmd, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE
+                *cmd,
+                stdout=asyncio.subprocess.PIPE,
+                stderr=asyncio.subprocess.PIPE,
+                env=self._env,
             )
             try:
                 stdout_task = asyncio.create_task(proc.stdout.readline())
@@ -466,23 +458,24 @@ MODULE_LICENSE("GPL");
         kernel_release = await self._kernel_release()
 
         extension = "" if format == "directory" else ("." + format)
-        package = output_dir / f"kernel-{kernel_release}.{self._arch}{extension}"
+        package = output_dir / f"kernel-{kernel_release}.{self._arch.name}{extension}"
 
         logger.info(
-            "packaging kernel %s from %s to %s",
+            "packaging kernel %s for %s from %s to %s",
             kernel_release,
+            self._arch.name,
             self._build_dir,
             package,
         )
 
         image_name = (
-            (await check_output("make", *make_args, "-s", "image_name"))
+            (await check_output("make", *make_args, "-s", "image_name", env=self._env))
             .decode()
             .strip()
         )
 
         with tempfile.TemporaryDirectory(
-            prefix="install.", dir=self._build_dir
+            prefix=package.name + ".tmp.", dir=package.parent
         ) as tmp_name:
             install_dir = Path(tmp_name)
             modules_dir = install_dir / "lib" / "modules" / kernel_release
@@ -495,25 +488,54 @@ MODULE_LICENSE("GPL");
                 "modules_install",
                 stdout=self._build_stdout,
                 stderr=self._build_stderr,
+                env=self._env,
             )
 
+            # `make modules_install` creates these as symlinks to the absolute
+            # path of the source directory. Delete them, make build a
+            # directory, and make source a symlink to build.
+            modules_build_dir = modules_dir / "build"
+            modules_source_dir = modules_dir / "source"
+            modules_build_dir.unlink()
+            modules_build_dir.mkdir()
+            modules_source_dir.unlink()
+            modules_source_dir.symlink_to("build")
+
             logger.info("copying vmlinux")
-            vmlinux = modules_dir / "vmlinux"
+            vmlinux = modules_build_dir / "vmlinux"
             await check_call(
-                "objcopy",
+                (os.environ if self._env is None else self._env).get(
+                    "CROSS_COMPILE", ""
+                )
+                + "objcopy",
                 "--remove-relocations=*",
                 self._build_dir / "vmlinux",
                 str(vmlinux),
+                env=self._env,
             )
             vmlinux.chmod(0o644)
 
             logger.info("copying vmlinuz")
             vmlinuz = modules_dir / "vmlinuz"
-            shutil.copy(self._build_dir / image_name, vmlinuz)
+            try:
+                shutil.copy(self._build_dir / image_name, vmlinuz)
+            except FileNotFoundError:
+                # Before Linux kernel commits 06995804b576 ("arm64: Use full
+                # path in KBUILD_IMAGE definition") and 152e6744ebfc ("arm: Use
+                # full path in KBUILD_IMAGE definition") (in v4.12), image_name
+                # may be relative to the architecture boot directory.
+                shutil.copy(
+                    self._build_dir
+                    / "arch"
+                    / self._arch.kernel_srcarch
+                    / "boot"
+                    / image_name,
+                    vmlinuz,
+                )
             vmlinuz.chmod(0o644)
 
-            self._copy_module_build(modules_dir)
-            await self._test_external_module_build(modules_dir)
+            self._copy_module_build(modules_build_dir)
+            await self._test_external_module_build(modules_build_dir)
 
             package.parent.mkdir(parents=True, exist_ok=True)
             if format == "directory":
@@ -541,7 +563,11 @@ MODULE_LICENSE("GPL");
                     raise CalledProcessError(zstd_returncode, zstd_cmd)
 
         logger.info(
-            "packaged kernel %s from %s to %s", kernel_release, self._build_dir, package
+            "packaged kernel %s for %s from %s to %s",
+            kernel_release,
+            self._arch.name,
+            self._build_dir,
+            package,
         )
         return package
 
@@ -585,15 +611,40 @@ async def main() -> None:
         help='package archive format, or "directory" for an unarchived directory',
         default=_PACKAGE_FORMATS[0],
     )
+    parser.add_argument(
+        "--patch",
+        action="store_true",
+        help="apply patches to kernel source tree",
+    )
+    parser.add_argument(
+        "-a",
+        "--architecture",
+        choices=sorted(ARCHITECTURES),
+        help="architecture to build for",
+        default=None if HOST_ARCHITECTURE is None else HOST_ARCHITECTURE.name,
+        required=HOST_ARCHITECTURE is None,
+    )
     parser.add_argument(
         "-f",
         "--flavor",
-        choices=[flavor.name for flavor in KERNEL_FLAVORS],
+        choices=KERNEL_FLAVORS,
         help="kernel configuration flavor. "
         + ". ".join(
-            [f"{flavor.name}: {flavor.description}" for flavor in KERNEL_FLAVORS]
+            [
+                f"{flavor.name}: {flavor.description}"
+                for flavor in KERNEL_FLAVORS.values()
+            ]
         ),
-        default=KERNEL_FLAVORS[0].name,
+        default=next(iter(KERNEL_FLAVORS)),
+    )
+    default_download_compiler_directory = Path("build/vmtest")
+    parser.add_argument(
+        "--download-compiler",
+        metavar="DIR",
+        nargs="?",
+        default=argparse.SUPPRESS,
+        type=Path,
+        help=f"download a compiler from {COMPILER_URL} to the given directory ({default_download_compiler_directory} by default) and use it to build",
     )
     parser.add_argument(
         "--dump-kconfig",
@@ -602,13 +653,26 @@ async def main() -> None:
     )
     args = parser.parse_args()
 
-    flavor = [flavor for flavor in KERNEL_FLAVORS if flavor.name == args.flavor][0]
+    arch = ARCHITECTURES[args.architecture]
+    flavor = KERNEL_FLAVORS[args.flavor]
 
     if args.dump_kconfig:
-        sys.stdout.write(kconfig(flavor))
+        sys.stdout.write(kconfig(arch, flavor))
         return
 
-    kbuild = KBuild(args.kernel_directory, args.build_directory, flavor, "x86_64")
+    if hasattr(args, "download_compiler"):
+        if args.download_compiler is None:
+            args.download_compiler = default_download_compiler_directory
+        downloaded = next(download(args.download_compiler, [DownloadCompiler(arch)]))
+        assert isinstance(downloaded, Compiler)
+        env = {**os.environ, **downloaded.env()}
+    else:
+        env = None
+
+    if args.patch:
+        await apply_patches(args.kernel_directory)
+
+    kbuild = KBuild(args.kernel_directory, args.build_directory, arch, flavor, env)
     await kbuild.build()
     if hasattr(args, "package"):
         await kbuild.package(args.package_format, args.package)
